@@ -21,6 +21,9 @@ class DeviceType(Enum):
     Hub = "hub"
     Fan = "fan"
     Thermostat = "thermostat"
+    Vacuum = "vacuum"
+    Chime = "chime"
+    Doorbell = "doorbell"
     Unknown = "unknown"
 
     @staticmethod
