@@ -3,13 +3,14 @@
 from __future__ import annotations
 
 import logging
-from typing import Any
+from typing import Any, cast
 
-from ..device import _DeviceInfo
+from ..device import DeviceInfo
 from ..device_type import DeviceType
 from ..module import Module
 from ..protocols.smartcamprotocol import _ChildCameraProtocolWrapper
 from ..smart import SmartChildDevice, SmartDevice
+from ..smart.smartdevice import ComponentsRaw
 from .modules import ChildDevice, DeviceModule
 from .smartcammodule import SmartCamModule
 
@@ -25,18 +26,21 @@ class SmartCamDevice(SmartDevice):
     @staticmethod
     def _get_device_type_from_sysinfo(sysinfo: dict[str, Any]) -> DeviceType:
         """Find type to be displayed as a supported device category."""
-        if (
-            sysinfo
-            and (device_type := sysinfo.get("device_type"))
-            and device_type.endswith("HUB")
-        ):
+        if not (device_type := sysinfo.get("device_type")):
+            return DeviceType.Unknown
+
+        if device_type.endswith("HUB"):
             return DeviceType.Hub
+
+        if "DOORBELL" in device_type:
+            return DeviceType.Doorbell
+
         return DeviceType.Camera
 
     @staticmethod
     def _get_device_info(
         info: dict[str, Any], discovery_info: dict[str, Any] | None
-    ) -> _DeviceInfo:
+    ) -> DeviceInfo:
         """Get model information for a device."""
         basic_info = info["getDeviceInfo"]["device_info"]["basic_info"]
         short_name = basic_info["device_model"]
@@ -44,7 +48,7 @@ class SmartCamDevice(SmartDevice):
         device_type = SmartCamDevice._get_device_type_from_sysinfo(basic_info)
         fw_version_full = basic_info["sw_version"]
         firmware_version, firmware_build = fw_version_full.split(" ", maxsplit=1)
-        return _DeviceInfo(
+        return DeviceInfo(
             short_name=basic_info["device_model"],
             long_name=long_name,
             brand="tapo",
@@ -62,23 +66,38 @@ class SmartCamDevice(SmartDevice):
         info = self._try_get_response(info_resp, "getDeviceInfo")
         self._info = self._map_info(info["device_info"])
 
-    def _update_children_info(self) -> None:
-        """Update the internal child device info from the parent info."""
+    def _update_internal_state(self, info: dict[str, Any]) -> None:
+        """Update the internal info state.
+
+        This is used by the parent to push updates to its children.
+        """
+        self._info = self._map_info(info)
+
+    async def _update_children_info(self) -> bool:
+        """Update the internal child device info from the parent info.
+
+        Return true if children added or deleted.
+        """
+        changed = False
         if child_info := self._try_get_response(
             self._last_update, "getChildDeviceList", {}
         ):
+            changed = await self._create_delete_children(
+                child_info, self._last_update["getChildDeviceComponentList"]
+            )
+
             for info in child_info["child_device_list"]:
-                child_id = info["device_id"]
+                child_id = info.get("device_id")
                 if child_id not in self._children:
-                    _LOGGER.debug(
-                        "Skipping child update for %s, probably unsupported device",
-                        child_id,
-                    )
+                    # _create_delete_children has already logged a message
                     continue
+
                 self._children[child_id]._update_internal_state(info)
 
+        return changed
+
     async def _initialize_smart_child(
-        self, info: dict, child_components: dict
+        self, info: dict, child_components_raw: ComponentsRaw
     ) -> SmartDevice:
         """Initialize a smart child device attached to a smartcam device."""
         child_id = info["device_id"]
@@ -93,11 +112,30 @@ class SmartCamDevice(SmartDevice):
         return await SmartChildDevice.create(
             parent=self,
             child_info=info,
-            child_components=child_components,
+            child_components_raw=child_components_raw,
             protocol=child_protocol,
             last_update=initial_response,
         )
 
+    async def _initialize_smartcam_child(
+        self, info: dict, child_components_raw: ComponentsRaw
+    ) -> SmartDevice:
+        """Initialize a smart child device attached to a smartcam device."""
+        child_id = info["device_id"]
+        child_protocol = _ChildCameraProtocolWrapper(child_id, self.protocol)
+
+        app_component_list = {
+            "app_component_list": child_components_raw["component_list"]
+        }
+        from .smartcamchild import SmartCamChild
+
+        return await SmartCamChild.create(
+            parent=self,
+            child_info=info,
+            child_components_raw=app_component_list,
+            protocol=child_protocol,
+        )
+
     async def _initialize_children(self) -> None:
         """Initialize children for hubs."""
         child_info_query = {
@@ -107,34 +145,22 @@ class SmartCamDevice(SmartDevice):
         resp = await self.protocol.query(child_info_query)
         self.internal_state.update(resp)
 
-        smart_children_components = {
-            child["device_id"]: {
-                comp["id"]: int(comp["ver_code"]) for comp in component_list
-            }
-            for child in resp["getChildDeviceComponentList"]["child_component_list"]
-            if (component_list := child.get("component_list"))
-            # Child camera devices will have a different component schema so only
-            # extract smart values.
-            and (first_comp := next(iter(component_list), None))
-            and isinstance(first_comp, dict)
-            and "id" in first_comp
-            and "ver_code" in first_comp
-        }
-        children = {}
-        for info in resp["getChildDeviceList"]["child_device_list"]:
-            if (
-                (category := info.get("category"))
-                and category in SmartChildDevice.CHILD_DEVICE_TYPE_MAP
-                and (child_id := info.get("device_id"))
-                and (child_components := smart_children_components.get(child_id))
-            ):
-                children[child_id] = await self._initialize_smart_child(
-                    info, child_components
-                )
-            else:
-                _LOGGER.debug("Child device type not supported: %s", info)
+    async def _try_create_child(
+        self, info: dict, child_components: dict
+    ) -> SmartDevice | None:
+        if not (category := info.get("category")):
+            return None
 
-        self._children = children
+        # Smart
+        if category in SmartChildDevice.CHILD_DEVICE_TYPE_MAP:
+            return await self._initialize_smart_child(info, child_components)
+        # Smartcam
+        from .smartcamchild import SmartCamChild
+
+        if category in SmartCamChild.CHILD_DEVICE_TYPE_MAP:
+            return await self._initialize_smartcam_child(info, child_components)
+
+        return None
 
     async def _initialize_modules(self) -> None:
         """Initialize modules based on component negotiation response."""
@@ -155,9 +181,6 @@ class SmartCamDevice(SmartDevice):
             for feat in module._module_features.values():
                 self._add_feature(feat)
 
-        for child in self._children.values():
-            await child._initialize_features()
-
     async def _query_setter_helper(
         self, method: str, module: str, section: str, params: dict | None = None
     ) -> dict:
@@ -165,12 +188,12 @@ class SmartCamDevice(SmartDevice):
 
         return res
 
-    async def _query_getter_helper(
-        self, method: str, module: str, sections: str | list[str]
-    ) -> Any:
-        res = await self.protocol.query({method: {module: {"name": sections}}})
-
-        return res
+    @staticmethod
+    def _parse_components(components_raw: ComponentsRaw) -> dict[str, int]:
+        return {
+            str(comp["name"]): int(comp["version"])
+            for comp in components_raw["app_component_list"]
+        }
 
     async def _negotiate(self) -> None:
         """Perform initialization.
@@ -181,34 +204,32 @@ class SmartCamDevice(SmartDevice):
         initial_query = {
             "getDeviceInfo": {"device_info": {"name": ["basic_info", "info"]}},
             "getAppComponentList": {"app_component": {"name": "app_component_list"}},
+            "getConnectionType": {"network": {"get_connection_type": {}}},
         }
         resp = await self.protocol.query(initial_query)
         self._last_update.update(resp)
         self._update_internal_info(resp)
 
-        self._components = {
-            comp["name"]: int(comp["version"])
-            for comp in resp["getAppComponentList"]["app_component"][
-                "app_component_list"
-            ]
-        }
+        self._components_raw = cast(
+            ComponentsRaw, resp["getAppComponentList"]["app_component"]
+        )
+        self._components = self._parse_components(self._components_raw)
 
         if "childControl" in self._components and not self.children:
             await self._initialize_children()
 
     def _map_info(self, device_info: dict) -> dict:
+        """Map the basic keys to the keys used by SmartDevices."""
         basic_info = device_info["basic_info"]
-        return {
-            "model": basic_info["device_model"],
-            "device_type": basic_info["device_type"],
-            "alias": basic_info["device_alias"],
-            "fw_ver": basic_info["sw_version"],
-            "hw_ver": basic_info["hw_version"],
-            "mac": basic_info["mac"],
-            "hwId": basic_info.get("hw_id"),
-            "oem_id": basic_info["oem_id"],
-            "device_id": basic_info["dev_id"],
+        mappings = {
+            "device_model": "model",
+            "device_alias": "alias",
+            "sw_version": "fw_ver",
+            "hw_version": "hw_ver",
+            "hw_id": "hwId",
+            "dev_id": "device_id",
         }
+        return {mappings.get(k, k): v for k, v in basic_info.items()}
 
     @property
     def is_on(self) -> bool:
@@ -228,7 +249,7 @@ class SmartCamDevice(SmartDevice):
     @property
     def device_type(self) -> DeviceType:
         """Return the device type."""
-        if self._device_type == DeviceType.Unknown:
+        if self._device_type == DeviceType.Unknown and self._info:
             self._device_type = self._get_device_type_from_sysinfo(self._info)
         return self._device_type
 
@@ -251,11 +272,16 @@ class SmartCamDevice(SmartDevice):
     def hw_info(self) -> dict:
         """Return hardware info for the device."""
         return {
-            "sw_ver": self._info.get("hw_ver"),
-            "hw_ver": self._info.get("fw_ver"),
+            "sw_ver": self._info.get("fw_ver"),
+            "hw_ver": self._info.get("hw_ver"),
             "mac": self._info.get("mac"),
             "type": self._info.get("type"),
             "hwId": self._info.get("hwId"),
             "dev_name": self.alias,
             "oemId": self._info.get("oem_id"),
         }
+
+    @property
+    def rssi(self) -> int | None:
+        """Return the device id."""
+        return self.modules[SmartCamModule.SmartCamDeviceModule].rssi
