@@ -121,9 +121,22 @@ DIMMERS = {
 }
 
 HUBS_SMART = {"H100", "KH100"}
-SENSORS_SMART = {"T310", "T315", "T300", "T100", "T110", "S200B", "S200D"}
+SENSORS_SMART = {
+    "T310",
+    "T315",
+    "T300",
+    "T100",
+    "T110",
+    "S200B",
+    "S200D",
+    "S210",
+    "S220",
+    "D100C",  # needs a home category?
+}
 THERMOSTATS_SMART = {"KE100"}
 
+VACUUMS_SMART = {"RV20"}
+
 WITH_EMETER_IOT = {"HS110", "HS300", "KP115", "KP125", *BULBS_IOT}
 WITH_EMETER_SMART = {"P110", "P110M", "P115", "KP125M", "EP25", "P304M"}
 WITH_EMETER = {*WITH_EMETER_IOT, *WITH_EMETER_SMART}
@@ -141,6 +154,7 @@ ALL_DEVICES_SMART = (
     .union(SENSORS_SMART)
     .union(SWITCHES_SMART)
     .union(THERMOSTATS_SMART)
+    .union(VACUUMS_SMART)
 )
 ALL_DEVICES = ALL_DEVICES_IOT.union(ALL_DEVICES_SMART)
 
@@ -325,13 +339,25 @@ device_smartcam = parametrize("devices smartcam", protocol_filter={"SMARTCAM"})
 camera_smartcam = parametrize(
     "camera smartcam",
     device_type_filter=[DeviceType.Camera],
-    protocol_filter={"SMARTCAM"},
+    protocol_filter={"SMARTCAM", "SMARTCAM.CHILD"},
 )
 hub_smartcam = parametrize(
     "hub smartcam",
     device_type_filter=[DeviceType.Hub],
     protocol_filter={"SMARTCAM"},
 )
+hubs = parametrize_combine([hubs_smart, hub_smartcam])
+doobell_smartcam = parametrize(
+    "doorbell smartcam",
+    device_type_filter=[DeviceType.Doorbell],
+    protocol_filter={"SMARTCAM", "SMARTCAM.CHILD"},
+)
+chime_smart = parametrize(
+    "chime smart",
+    device_type_filter=[DeviceType.Chime],
+    protocol_filter={"SMART"},
+)
+vacuum = parametrize("vacuums", device_type_filter=[DeviceType.Vacuum])
 
 
 def check_categories():
@@ -348,8 +374,11 @@ def check_categories():
         + hubs_smart.args[1]
         + sensors_smart.args[1]
         + thermostats_smart.args[1]
+        + chime_smart.args[1]
         + camera_smartcam.args[1]
+        + doobell_smartcam.args[1]
         + hub_smartcam.args[1]
+        + vacuum.args[1]
     )
     diffs: set[FixtureInfo] = set(FIXTURE_DATA) - set(categorized_fixtures)
     if diffs:
@@ -367,7 +396,7 @@ check_categories()
 def device_for_fixture_name(model, protocol):
     if protocol in {"SMART", "SMART.CHILD"}:
         return SmartDevice
-    elif protocol == "SMARTCAM":
+    elif protocol in {"SMARTCAM", "SMARTCAM.CHILD"}:
         return SmartCamDevice
     else:
         for d in STRIPS_IOT:
@@ -420,11 +449,20 @@ async def get_device_for_fixture(
     d = device_for_fixture_name(fixture_data.name, fixture_data.protocol)(
         host="127.0.0.123"
     )
+
+    # smart child devices sometimes check _is_hub_child which needs a parent
+    # of DeviceType.Hub
+    class DummyParent:
+        device_type = DeviceType.Hub
+
+    if fixture_data.protocol in {"SMARTCAM.CHILD"}:
+        d._parent = DummyParent()
+
     if fixture_data.protocol in {"SMART", "SMART.CHILD"}:
         d.protocol = FakeSmartProtocol(
             fixture_data.data, fixture_data.name, verbatim=verbatim
         )
-    elif fixture_data.protocol == "SMARTCAM":
+    elif fixture_data.protocol in {"SMARTCAM", "SMARTCAM.CHILD"}:
         d.protocol = FakeSmartCamProtocol(
             fixture_data.data, fixture_data.name, verbatim=verbatim
         )
